/*
 *   Copyright (c) International Business Machines  Corp., 2000
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or 
 *   (at your option) any later version.
 * 
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software 
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *	fssubs.h
 *
 */

#ifndef _H_UJFS_FSSUBS
#define _H_UJFS_FSSUBS

#define	PAGESIZE	4096
#define RC_OK       0  

/*
 * logical volume parameter
 */
extern	HFILE	LVHandle;
typedef struct {
	s64	 LVSize;	/* LV size in pbsize        */
	s32	 pbsize;	/* device block/sector size */
	s32	 l2pbsize;	/* log2 of pbsize           */
	u8	 LVNumber;	/* vpfsi.vpi_drive          */
} LV_t;

extern LV_t 	LVMount;
extern LV_t	*lvMount;

/*
 * file system parameter
 */
typedef struct {
	s64 	FSSize;		/* FS size in bsize              */

	s32 	bsize;		/* FS logical block size in byte */
	s32 	l2bsize;

	s32 	l2bfactor;	/* log2 (bsize/pbsize)           */

	s32 	nbperpage;
	s32 	l2nbperpage;

	s32 	AGSize;		/* FS AG size in bsize           */
	s32 	nAG;		/* number of AGs                 */

	s64 	LOGSize;	/* log size in bsize             */
} FS_t;

extern FS_t	FSMount;
extern FS_t	*fsMount;

/*
 * block allocation map
 */
extern dinode_t	DIBMap;
extern dinode_t	*diBMap;

typedef struct {
	dbmap_t		ctl;
	dmapctl_t	l2;
	dmapctl_t	l1;
	dmapctl_t	l0;
} BMap_t;
extern  BMap_t	BMap;
extern  BMap_t	*bMap;

/*
 * inode allocation map
 */
extern dinode_t	DIIMap;
extern dinode_t	*diIMap;

typedef struct {
	dinomap_t	ctl;
} IMap_t;
extern IMap_t	IMap;
extern IMap_t	*iMap;

/* imap xtree sequential read control */
extern s32	IAGNumber;
extern s64	IMapXtreeLMLeaf;	/* imap xtree leftmost leaf xaddr */
extern xtpage_t	IMapXtreeLeaf;  /* imap xtree leaf buffer */
typedef struct {
	xtpage_t	*leaf;	/* current imap leaf xtpage under scan */
	s32	 index;	/* current xad entry index in iMapXtreeLeaf */ 
	s32	 page;	/* iag number to read within current imap extent*/ 
} IMapXtree_t;
extern IMapXtree_t	IMapXtree;

/*
 *	buffer pool
 */
typedef struct {
	char  page[PAGESIZE];
} page_t; 

typedef struct bufhdr {
	struct bufhdr  *b_next;	/* 4: */
	char   *b_data;		    /* 4: */
} buf_t;				    /* (8) */

	
/*
 * function prototypes
 */
s32 openLV(char *LVName);
s32 openFS(void);
void closeFS(void);
s32 readBMapGCP(BMap_t *bMap);
s32 readBMapLCP(s64 bn, s32 level, dmapctl_t *lcp);
s32 readBMap(s64 start_dmap, s32 ndmaps, dmap_t *dmap);
s32 readIMapGCPSequential(IMap_t *iMap, iag_t *iag);
s32 readIMapSequential(iag_t *iag);
s32 xtLookup(dinode_t *dip, s64 xoff, s64 *xaddr, s32 *xlen, u32 flag);
s32 xtLMLeaf(dinode_t *dip, xtpage_t *xpp);
s32 fscntl(u32 cmd, void *pList, ULONG *pListLen, 
	       void *pData, ULONG *pDataLen);
s32 pRead(FS_t *fsMount, s64 xaddr, s32 xlen, void *p);
s32 bRead(FS_t *fsMount, s64 xaddr, s32 xlen, buf_t **bpp);
s32 bRawRead(u32 LVHandle, s64 off, s32 len, buf_t **bpp);
void bRelease(buf_t *bp);
int Is_Device_Mounted ( char * );

#endif	/* _H_UJFS_FSSUBS */
